@@ -292,7 +292,7 @@ fn full_shortcut_list(help: &ModeInfo) -> LinePart {
         _ => {
             let mut line_part = LinePart::default();
             for (i, (letter, description)) in help.keybinds.iter().enumerate() {
-                let shortcut = full_length_shortcut(i == 0, &letter, &description, help.palette);
+                let shortcut = full_length_shortcut(i == 0, letter, description, help.palette);
                 line_part.len += shortcut.len;
                 line_part.part = format!("{}{}", line_part.part, shortcut,);
             }
@@ -311,7 +311,7 @@ fn shortened_shortcut_list(help: &ModeInfo) -> LinePart {
         _ => {
             let mut line_part = LinePart::default();
             for (i, (letter, description)) in help.keybinds.iter().enumerate() {
-                let shortcut = first_word_shortcut(i == 0, &letter, &description, help.palette);
+                let shortcut = first_word_shortcut(i == 0, letter, description, help.palette);
                 line_part.len += shortcut.len;
                 line_part.part = format!("{}{}", line_part.part, shortcut,);
             }
@@ -344,7 +344,7 @@ fn best_effort_shortcut_list(help: &ModeInfo, max_len: usize) -> LinePart {
         _ => {
             let mut line_part = LinePart::default();
             for (i, (letter, description)) in help.keybinds.iter().enumerate() {
-                let shortcut = first_word_shortcut(i == 0, &letter, &description, help.palette);
+                let shortcut = first_word_shortcut(i == 0, letter, description, help.palette);
                 if line_part.len + shortcut.len + MORE_MSG.chars().count() > max_len {
                     // TODO: better
                     line_part.part = format!("{}{}", line_part.part, MORE_MSG);
