@@ -15,23 +15,26 @@ use std::path::PathBuf;
 use std::sync::{Arc, Mutex, RwLock};
 use std::thread;
 use wasmer::Store;
-use zellij_tile::data::{Event, InputMode, PluginCapabilities};
+use zellij_tile::data::{Event, Palette, PluginCapabilities};
 
 use crate::{
     os_input_output::ServerOsApi,
     pty::{pty_thread_main, Pty, PtyInstruction},
     screen::{screen_thread_main, ScreenInstruction},
     thread_bus::{Bus, ThreadSenders},
-    ui::layout::Layout,
     wasm_vm::{wasm_thread_main, PluginInstruction},
 };
 use route::route_thread_main;
 use zellij_utils::{
-    channels,
-    channels::{ChannelWithContext, SenderWithContext},
+    channels::{self, ChannelWithContext, SenderWithContext},
     cli::CliArgs,
     errors::{ContextType, ErrorInstruction, ServerContext},
-    input::{get_mode_info, options::Options},
+    input::{
+        command::{RunCommand, TerminalAction},
+        get_mode_info,
+        layout::Layout,
+        options::Options,
+    },
     ipc::{ClientAttributes, ClientToServerMsg, ExitReason, ServerToClientMsg},
     setup::get_default_data_dir,
 };
@@ -39,23 +42,23 @@ use zellij_utils::{
 /// Instructions related to server-side application
 #[derive(Debug, Clone)]
 pub(crate) enum ServerInstruction {
-    NewClient(ClientAttributes, Box<CliArgs>, Box<Options>),
+    NewClient(ClientAttributes, Box<CliArgs>, Box<Options>, Option<Layout>),
     Render(Option<String>),
     UnblockInputThread,
     ClientExit,
     Error(String),
     DetachSession,
-    AttachClient(ClientAttributes, bool),
+    AttachClient(ClientAttributes, bool, Options),
 }
 
 impl From<ClientToServerMsg> for ServerInstruction {
     fn from(instruction: ClientToServerMsg) -> Self {
         match instruction {
-            ClientToServerMsg::NewClient(attrs, opts, options) => {
-                ServerInstruction::NewClient(attrs, opts, options)
+            ClientToServerMsg::NewClient(attrs, opts, options, layout) => {
+                ServerInstruction::NewClient(attrs, opts, options, layout)
             }
-            ClientToServerMsg::AttachClient(attrs, force) => {
-                ServerInstruction::AttachClient(attrs, force)
+            ClientToServerMsg::AttachClient(attrs, force, options) => {
+                ServerInstruction::AttachClient(attrs, force, options)
             }
             _ => unreachable!(),
         }
@@ -85,6 +88,8 @@ impl ErrorInstruction for ServerInstruction {
 pub(crate) struct SessionMetaData {
     pub senders: ThreadSenders,
     pub capabilities: PluginCapabilities,
+    pub palette: Palette,
+    pub default_shell: Option<TerminalAction>,
     screen_thread: Option<thread::JoinHandle<()>>,
     pty_thread: Option<thread::JoinHandle<()>>,
     wasm_thread: Option<thread::JoinHandle<()>>,
@@ -113,6 +118,8 @@ pub fn start_server(os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
     daemonize::Daemonize::new()
         .working_directory(std::env::current_dir().unwrap())
         .umask(0o077)
+        // FIXME: My cherished `dbg!` was broken, so this is a hack to bring it back
+        //.stderr(std::fs::File::create("dbg.log").unwrap())
         .start()
         .expect("could not daemonize the server process");
 
@@ -206,27 +213,36 @@ pub fn start_server(os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
         let (instruction, mut err_ctx) = server_receiver.recv().unwrap();
         err_ctx.add_call(ContextType::IPCServer((&instruction).into()));
         match instruction {
-            ServerInstruction::NewClient(client_attributes, opts, config_options) => {
+            ServerInstruction::NewClient(client_attributes, opts, config_options, layout) => {
                 let session = init_session(
                     os_input.clone(),
                     opts,
-                    config_options,
+                    config_options.clone(),
                     to_server.clone(),
                     client_attributes,
                     session_state.clone(),
+                    layout,
                 );
                 *session_data.write().unwrap() = Some(session);
                 *session_state.write().unwrap() = SessionState::Attached;
+
+                let default_shell = config_options.default_shell.map(|shell| {
+                    TerminalAction::RunCommand(RunCommand {
+                        command: shell,
+                        ..Default::default()
+                    })
+                });
+
                 session_data
                     .read()
                     .unwrap()
                     .as_ref()
                     .unwrap()
                     .senders
-                    .send_to_pty(PtyInstruction::NewTab)
+                    .send_to_pty(PtyInstruction::NewTab(default_shell.clone()))
                     .unwrap();
             }
-            ServerInstruction::AttachClient(attrs, _) => {
+            ServerInstruction::AttachClient(attrs, _, options) => {
                 *session_state.write().unwrap() = SessionState::Attached;
                 let rlock = session_data.read().unwrap();
                 let session_data = rlock.as_ref().unwrap();
@@ -234,8 +250,9 @@ pub fn start_server(os_input: Box<dyn ServerOsApi>, socket_path: PathBuf) {
                     .senders
                     .send_to_screen(ScreenInstruction::TerminalResize(attrs.position_and_size))
                     .unwrap();
+                let default_mode = options.default_mode.unwrap_or_default();
                 let mode_info =
-                    get_mode_info(InputMode::Normal, attrs.palette, session_data.capabilities);
+                    get_mode_info(default_mode, attrs.palette, session_data.capabilities);
                 session_data
                     .senders
                     .send_to_screen(ScreenInstruction::ChangeMode(mode_info.clone()))
@@ -300,9 +317,15 @@ fn init_session(
     to_server: SenderWithContext<ServerInstruction>,
     client_attributes: ClientAttributes,
     session_state: Arc<RwLock<SessionState>>,
+    layout: Option<Layout>,
 ) -> SessionMetaData {
     let (to_screen, screen_receiver): ChannelWithContext<ScreenInstruction> = channels::unbounded();
     let to_screen = SenderWithContext::new(to_screen);
+
+    let (to_screen_bounded, bounded_screen_receiver): ChannelWithContext<ScreenInstruction> =
+        channels::bounded(50);
+    let to_screen_bounded = SenderWithContext::new(to_screen_bounded);
+
     let (to_plugin, plugin_receiver): ChannelWithContext<PluginInstruction> = channels::unbounded();
     let to_plugin = SenderWithContext::new(to_plugin);
     let (to_pty, pty_receiver): ChannelWithContext<PtyInstruction> = channels::unbounded();
@@ -315,18 +338,12 @@ fn init_session(
         arrow_fonts: config_options.simplified_ui,
     };
 
-    // Don't use default layouts in tests, but do everywhere else
-    #[cfg(not(any(feature = "test", test)))]
-    let default_layout = Some(PathBuf::from("default"));
-    #[cfg(any(feature = "test", test))]
-    let default_layout = None;
-    let layout_path = opts.layout_path;
-    let maybe_layout = opts
-        .layout
-        .as_ref()
-        .map(|p| Layout::from_dir(&p, &data_dir))
-        .or_else(|| layout_path.map(|p| Layout::new(&p)))
-        .or_else(|| default_layout.map(|p| Layout::from_dir(&p, &data_dir)));
+    let default_shell = config_options.default_shell.clone().map(|command| {
+        TerminalAction::RunCommand(RunCommand {
+            command,
+            ..Default::default()
+        })
+    });
 
     let pty_thread = thread::Builder::new()
         .name("pty".to_string())
@@ -334,7 +351,7 @@ fn init_session(
             let pty = Pty::new(
                 Bus::new(
                     vec![pty_receiver],
-                    Some(&to_screen),
+                    Some(&to_screen_bounded),
                     None,
                     Some(&to_plugin),
                     Some(&to_server),
@@ -343,7 +360,7 @@ fn init_session(
                 opts.debug,
             );
 
-            move || pty_thread_main(pty, maybe_layout)
+            move || pty_thread_main(pty, layout)
         })
         .unwrap();
 
@@ -351,7 +368,7 @@ fn init_session(
         .name("screen".to_string())
         .spawn({
             let screen_bus = Bus::new(
-                vec![screen_receiver],
+                vec![screen_receiver, bounded_screen_receiver],
                 None,
                 Some(&to_pty),
                 Some(&to_plugin),
@@ -394,8 +411,11 @@ fn init_session(
             to_pty: Some(to_pty),
             to_plugin: Some(to_plugin),
             to_server: None,
+            should_silently_fail: false,
         },
         capabilities,
+        default_shell,
+        palette: client_attributes.palette,
         screen_thread: Some(screen_thread),
         pty_thread: Some(pty_thread),
         wasm_thread: Some(wasm_thread),
