@@ -1,4 +1,5 @@
 use serde::{Deserialize, Serialize};
+use std::str::FromStr;
 use strum_macros::{EnumDiscriminants, EnumIter, EnumString, ToString};
 
 #[derive(Debug, Copy, Clone, PartialEq, Eq, Hash, Serialize, Deserialize)]
@@ -71,13 +72,13 @@ impl Default for InputMode {
 }
 
 #[derive(Clone, Copy, Debug, Serialize, Deserialize, PartialEq, Eq, Hash)]
-pub enum Theme {
+pub enum ThemeHue {
     Light,
     Dark,
 }
-impl Default for Theme {
-    fn default() -> Theme {
-        Theme::Dark
+impl Default for ThemeHue {
+    fn default() -> ThemeHue {
+        ThemeHue::Dark
     }
 }
 
@@ -92,6 +93,24 @@ impl Default for PaletteColor {
     }
 }
 
+impl FromStr for InputMode {
+    type Err = Box<dyn std::error::Error>;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        match s {
+            "normal" => Ok(InputMode::Normal),
+            "resize" => Ok(InputMode::Resize),
+            "locked" => Ok(InputMode::Locked),
+            "pane" => Ok(InputMode::Pane),
+            "tab" => Ok(InputMode::Tab),
+            "scroll" => Ok(InputMode::Scroll),
+            "renametab" => Ok(InputMode::RenameTab),
+            "session" => Ok(InputMode::Session),
+            e => Err(e.to_string().into()),
+        }
+    }
+}
+
 #[derive(Clone, Copy, Debug, Serialize, Deserialize, PartialEq, Eq, Hash)]
 pub enum PaletteSource {
     Default,
@@ -105,7 +124,7 @@ impl Default for PaletteSource {
 #[derive(Clone, Copy, Debug, Serialize, Deserialize, PartialEq, Eq, Hash, Default)]
 pub struct Palette {
     pub source: PaletteSource,
-    pub theme: Theme,
+    pub theme_hue: ThemeHue,
     pub fg: PaletteColor,
     pub bg: PaletteColor,
     pub black: PaletteColor,
